<?php

namespace FiloBlu\Design\Helper;

class Data extends \Magento\Framework\App\Helper\AbstractHelper {

    const CONTENT_DIR_IMAGE = 'contentmanager/content/';

    protected $_mathRandom;
    protected $_storeManager;
    protected $_categoryRepository;
    protected $_productRepository;
    protected $_resourceModelProduct;
    protected $_filesystem;
    protected $_imageFactory;
    protected $_timezoneInterface;
    protected $_directory_list;
    protected $_filterProvider;
    protected $_productImageHelper;

    public function __construct(
    \Magento\Framework\App\Helper\Context $context, \Magento\Store\Model\StoreManagerInterface $storeManager, \Magento\Catalog\Model\CategoryRepository $categoryRepository, \Magento\Catalog\Model\ProductRepository $productRepository, \Magento\Catalog\Model\ResourceModel\Product $resourceModelProduct, \Magento\Framework\Filesystem $filesystem, \Magento\Framework\Image\AdapterFactory $imageFactory, \Magento\Framework\Math\Random $mathRandom, \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface, \Magento\Framework\App\Filesystem\DirectoryList $directory_list, \Magento\Cms\Model\Template\FilterProvider $filterProvider, \Magento\Catalog\Helper\Image $productImageHelper
    ) {
        $this->_storeManager = $storeManager;
        parent::__construct($context);
        $this->_categoryRepository = $categoryRepository;
        $this->_productRepository = $productRepository;
        $this->_resourceModelProduct = $resourceModelProduct;
        $this->_filesystem = $filesystem;
        $this->_imageFactory = $imageFactory;
        $this->_mathRandom = $mathRandom;
        $this->_urlInterface = $context->getUrlBuilder();
        $this->_timezoneInterface = $timezoneInterface;
        $this->_directory_list = $directory_list;
        $this->_filterProvider = $filterProvider;
        $this->_productImageHelper = $productImageHelper;
    }

    public function getConfig($config_path, $section = 'filoblu_design_section') {
        return $this->scopeConfig->getValue(
                        $section . '/' . $config_path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    public function getContentDirImage() {
        return SELF::CONTENT_DIR_IMAGE;
    }

    public function getUrlMedia($path) {
        return $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . $path;
    }

    public function getPathMedia($path) {
        return $this->_filesystem->getDirectoryRead(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA)->getAbsolutePath($path);
    }

    public function getTransparentImage() {
        return $this->getUrlMedia('filoblu/lazyload/' . $this->getConfig('js_library/lazyload_image_transparent'));
    }

    public function getLoadingImage() {
        return $this->getUrlMedia('filoblu/lazyload/' . $this->getConfig('js_library/lazyload_image_loading'));
    }

    public function getImageResized($image, $width = null, $height = null) {
        $absolutePath = $this->getPathMedia($image);
        $destination = 'contentmanager/content/resized/' . $width . 'x' . $height . '/' . $image;

        if (is_file($this->getPathMedia($destination))) {
            return $this->getUrlMedia($destination);
        }

        if (is_file($absolutePath)) {

            $imageResize = $this->_imageFactory->create();
            $imageResize->open($absolutePath);
            $imageResize->constrainOnly(true);
            $imageResize->keepTransparency(true);
            $imageResize->keepFrame(false);
            $imageResize->keepAspectRatio(true);
            $imageResize->resize($width, $height);
            $imageResize->quality($this->getConfig('images_settings/image_quality_resize'));
            //save image
            $imageResize->save($this->getPathMedia($destination));

            return $this->getUrlMedia($destination);
        } else {
            return "";
        }
    }

    public function getImgHtml(
    $imgPathDesktop, $imgPathTablet, $imgPathMobile, $resizeWidthDesktop, $resizeHeightDesktop, $resizeWidthTablet, $resizeHeightTablet, $resizeWidthMobile, $resizeHeightMobile, $dataForceLoad, $lazyloadContainer, $class, $alt, $idImage, $createBgImageDiv, $dataMageInit = null) {

        if (!is_file($this->getPathMedia($imgPathTablet))) {
            $imgPathTablet = null;
        }

        if (!is_file($this->getPathMedia($imgPathMobile))) {
            $imgPathMobile = null;
        }

        if ($imgPathTablet == $imgPathDesktop) {
            $imgPathTablet = null;
        }

        if ($imgPathMobile == $imgPathDesktop) {
            $imgPathMobile = null;
        }

        // Desktop and default data
        $imgWidthDesktop = null;
        $imgHeightDesktop = null;
        $imgFullPathDesktop = $this->getPathMedia($imgPathDesktop);

        if ($lazyloadContainer && is_file($imgFullPathDesktop)) {
            $size = getimagesize($imgFullPathDesktop);
            $imgWidthDesktop = $size[0];
            $imgHeightDesktop = $size[1];
        }

        $desktopData = ''
                . 'data-src="' . $this->getImageResized($imgPathDesktop, $resizeWidthDesktop, $resizeHeightDesktop) . '"'
                . 'data-original-width="' . $imgWidthDesktop . '"'
                . 'data-original-height="' . $imgHeightDesktop . '"';

        # Tablet data
        $tabletData = '';

        if (!$imgPathTablet && ($resizeWidthTablet && ($resizeWidthTablet != $resizeWidthDesktop) || $resizeHeightTablet && ($resizeHeightTablet != $resizeHeightDesktop))) {
            $imgPathTablet = $imgPathDesktop;
        }

        $imgWidthTablet = null;
        $imgHeightTablet = null;
        if ($imgPathTablet) {
            $imgFullPathTablet = $this->getPathMedia($imgPathTablet);

            if ($lazyloadContainer && is_file($imgFullPathTablet)) {
                $size = getimagesize($imgFullPathTablet);
                $imgWidthTablet = $size[0];
                $imgHeightTablet = $size[1];
            }

            $dataSrcTablet = $this->getImageResized($imgPathTablet, $resizeWidthTablet, $resizeHeightTablet);
            $tabletData = ''
                    . 'data-src-tablet="' . $dataSrcTablet . '"'
                    . 'data-src-tablet_retina="' . $dataSrcTablet . '"'
                    . 'data-original-width-tablet="' . $imgWidthTablet . '"'
                    . 'data-original-height-tablet="' . $imgHeightTablet . '"'
                    . 'data-original-width-tablet_retina="' . $imgWidthTablet . '"'
                    . 'data-original-height-tablet_retina="' . $imgHeightTablet . '"';
        }

        # Mobile data
        $mobileData = '';

        if (!$imgPathMobile && ($resizeWidthMobile && ($resizeWidthMobile != $resizeWidthDesktop) || $resizeHeightMobile && ($resizeHeightMobile != $resizeHeightDesktop))) {
            $imgPathMobile = $imgPathDesktop;
        }

        $imgWidthMobile = null;
        $imgHeightMobile = null;
        if ($imgPathMobile) {
            $imgFullPathMobile = $this->getPathMedia($imgPathMobile);

            if ($lazyloadContainer && is_file($imgFullPathMobile)) {
                $size = getimagesize($imgFullPathMobile);
                $imgWidthMobile = $size[0];
                $imgHeightMobile = $size[1];
            }

            $dataSrcMobile = $this->getImageResized($imgPathMobile, $resizeWidthMobile, $resizeHeightMobile);
            $mobileData = ''
                    . 'data-src-mobile="' . $dataSrcMobile . '"'
                    . 'data-src-mobile_retina="' . $dataSrcMobile . '"'
                    . 'data-original-width-mobile="' . $imgWidthMobile . '"'
                    . 'data-original-height-mobile="' . $imgHeightMobile . '"'
                    . 'data-original-width-mobile_retina="' . $imgWidthMobile . '"'
                    . 'data-original-height-mobile_retina="' . $imgHeightMobile . '"';
        }


        $img = '';
        $rndId = $this->getRandomString(10);

        if ($lazyloadContainer) {
            $img .= '<span class="lazyload-container" id="lazyload-container_'.$rndId.'">';
        }

        $dataForceLoadData = ($dataForceLoad ? 'data-force-load="true"' : '');
        $bgElementData = '';
        if ($createBgImageDiv) {
            if (!$dataForceLoadData) {
                $dataForceLoadData = 'data-force-load="true"';
            }

            $class .= " no-display";
            $bgElementData = 'data-bg-element-id="#' . $rndId . '"';
            $img .= '<div class="lazyload bg-image" id="' . $rndId . '" style="background-image:url(' . $this->getLoadingImage() . ')"></div>';
        }

        if ($dataMageInit) {
            $dataMageInit = "data-mage-init='" . $dataMageInit . "'";
        }

        $img .= '<img
                        src="' . ($lazyloadContainer ? $this->getTransparentImage() : $this->getLoadingImage()) . '"
                        ' . $bgElementData . '
                        ' . $desktopData . '
                        ' . $tabletData . '
                        ' . $mobileData . '
                        ' . $dataForceLoadData . '
                        ' . $dataMageInit . '
                        class="lazyload ' . $class . '"
                        id="' . $idImage . '"
                        alt="' . $alt . '"
                        />';

        $_breakpointConfig = unserialize($this->getConfig('js_library/breakpoint_config'));
        $_mediaQueries = NULL;

        foreach ($_breakpointConfig as $conf):

            if($conf["breakpointtype"] == 'mobile' || $conf["breakpointtype"] == 'mobile_retina' && $imgPathMobile){
                $_mediaQueries .= '
                    @media only screen and '.$conf["breakpointmediaquery"].'{
                        #lazyload-container_'.$rndId.'{
                            padding-bottom: ' . $this->calculateLazyLoadPaddingBottom($mobileData ? $imgWidthMobile : $imgWidthDesktop , $mobileData ? $imgHeightMobile : $imgHeightDesktop) . ';
                        }
                    }
                ';

            }

            if($conf["breakpointtype"] == 'tablet' || $conf["breakpointtype"] == 'tablet_retina' && $imgPathTablet){

                $_mediaQueries .= '
                    @media only screen and '.$conf["breakpointmediaquery"].'{
                        #lazyload-container_'.$rndId.'{
                            padding-bottom: ' . $this->calculateLazyLoadPaddingBottom($tabletData ? $imgWidthTablet : $imgWidthDesktop, $tabletData ? $imgHeightTablet : $imgHeightDesktop) . ';
                        }
                    }
                ';

            }

            if($conf["breakpointtype"] == 'desktop'){

                $_mediaQueries .= '
                    @media only screen and '.$conf["breakpointmediaquery"].'{
                        #lazyload-container_'.$rndId.'{
                            padding-bottom: ' . $this->calculateLazyLoadPaddingBottom($imgWidthDesktop, $imgHeightDesktop) . ';
                        }
                    }
                ';

            }



        endforeach;

        if ($lazyloadContainer) {
            $img .= '</span>';
            $img .= '<style>
                            '.$_mediaQueries.'
                    </style>';
        }

        return $img;
    }

    public function getCategoryUrlById($categoryId) {
        return ( $categoryId ) ? $this->_categoryRepository->get($categoryId)->getUrl() : null;
    }

    public function getProductUrlBySku($productSku) {
        $productId = $this->_resourceModelProduct->getIdBySku($productSku);
        if ($productId) {
            $product = $this->_productRepository->getById($productId);
            return $product->getUrlModel()->getUrl($product);
        } else {
            return null;
        }
    }

    public function getProductUrlById($productId) {
        if ($productId) {
            $product = $this->_productRepository->getById($productId);
            return $product->getUrlModel()->getUrl($product);
        } else {
            return null;
        }
    }

    public function getProductDataBySku($productSku) {
        $productId = $this->_resourceModelProduct->getIdBySku($productSku);
        if ($productId) {
            $product = $this->_productRepository->getById($productId);
            return $product;
        } else {
            return null;
        }
    }

    public function getProductDataById($productId) {
        if ($productId) {
            $product = $this->_productRepository->getById($productId);
            return $product;
        } else {
            return null;
        }
    }

    public function getProductInstanceBySku($productSku) {
        $productId = $this->_resourceModelProduct->getIdBySku($productSku);
        if ($productId) {
            $product = $this->_productRepository->getById($productId);
            return $product;
        } else {
            return null;
        }
    }

    public function getProductInstanceById($productId) {
        if ($productId) {
            $product = $this->_productRepository->getById($productId);
            return $product;
        } else {
            return null;
        }
    }

    public function getRandomString($length) {
        return $this->_mathRandom->getRandomString($length);
    }

    public function cleanHtmlForSharing($text, $type = 'standard') {

        $encoding = mb_detect_encoding($text, 'UTF-8, ISO-8859-1');
        if ($type == 'standard') {
            $outp_chr = array('...', "'", "'", '"', '"', '•', '-', '-');
        } elseif ($type == 'reference') {
            $outp_chr = array('&#8230;', '&#8216;', '&#8217;', '&#8220;', '&#8221;', '&#8226;', '&#8211;', '&#8212;'); // decimal numerical character references
        }
        $utf8_chr = array("\xe2\x80\xa6", "\xe2\x80\x98", "\xe2\x80\x99", "\xe2\x80\x9c", "\xe2\x80\x9d", '\xe2\x80\xa2', "\xe2\x80\x93", "\xe2\x80\x94"); // UTF-8 hex characters
        $winc_chr = array(chr(133), chr(145), chr(146), chr(147), chr(148), chr(149), chr(150), chr(151)); // ASCII characters (found in Windows-1252)
        $text = str_replace($utf8_chr, $outp_chr, $text);
        $text = str_replace($winc_chr, $outp_chr, $text);
        $text = mb_convert_encoding($text, 'UTF-8', $encoding);

        $text = preg_replace('/\s+/', ' ', addslashes($text));

        return $text;
    }

    public function getFilteredContent($string) {
        return $this->_filterProvider->getBlockFilter()->setStoreId($this->_storeManager->getStore()->getId())->filter($string);
    }

    public function getContentUrl($linkType, $linkCustom, $linkCategory, $linkProduct) {
        $_url = null;

        switch ($linkType) {
            case 'default':
            case 'none':
                $_url = '';
                break;
            case 'custom':
                $_url = $this->getFilteredContent($linkCustom);
                break;

            case 'category':
                $_url = $this->getCategoryUrlById($linkCategory);
                break;

            case 'product':
                $_url = $this->getProductUrlBySku($linkProduct);
                break;
        }

        return trim($_url);
    }

    public function deleteDirectory($path) {
        $this->_filesystem->getDirectoryWrite(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA)->delete($path);
    }

    public function cleanContentManagerImages() {
        $this->deleteDirectory("contentmanager/content/resized/");
    }


    public function getImgListingHtml(
    $imgUrlDesktop, $imgOriginalWidth, $imgOriginalHeight, $dataForceLoad, $lazyloadContainer, $class, $alt, $idImage, $dataMageInit = null, $lowQualityPreview = null) {

        $type = '';
        $_product = null;
        $width = '';
        $height = '';
        $lowQualityUrl = '';
        $rndId = $this->getRandomString(10);

        /* if $lowQuality Preview */
        if ($lowQualityPreview && is_array($lowQualityPreview)) {

            if (array_key_exists('type', $lowQualityPreview)) {
                $type = $lowQualityPreview['type'];
            }
            if (array_key_exists('product', $lowQualityPreview)) {
                $_product = $lowQualityPreview['product'];
            }
            if (array_key_exists('width', $lowQualityPreview) && array_key_exists('height', $lowQualityPreview)) {
                $width = $lowQualityPreview['width'];
                $height = $lowQualityPreview['height'];
            } elseif (array_key_exists('width', $lowQualityPreview) && !array_key_exists('height', $lowQualityPreview)) {

                $width = $lowQualityPreview['width'];
                $height = $this->calculateLowQualityImageProportion($imgOriginalWidth, $imgOriginalHeight, $lowQualityPreview['width'], 'width');
            } elseif (array_key_exists('height', $lowQualityPreview) && !array_key_exists('width', $lowQualityPreview)) {
                $height = $lowQualityPreview['height'];
                $width = $this->calculateLowQualityImageProportion($imgOriginalWidth, $imgOriginalHeight, $lowQualityPreview['height'], 'height');
            }


            if ($type && $_product) {
                $lowQualityUrl = $this->_productImageHelper->init($_product, $type)
                                ->constrainOnly(false)
                                ->keepAspectRatio(TRUE)
                                ->keepTransparency(TRUE)
                                ->keepFrame(true)
                                ->resize($width, $height)->getUrl();
            }
        }

        /**/

        if ($imgUrlDesktop == '') {
            $imgUrlDesktop = null;
        }

        $desktopData = ''
                . 'data-src="' . $imgUrlDesktop . '"'
                . 'data-original-width="' . $imgOriginalWidth . '"'
                . 'data-original-height="' . $imgOriginalHeight . '"';


        $img = '';

        if ($lazyloadContainer && !$lowQualityPreview) {

            $img .= '<span class="lazyload-container" id="lazyload-container_'.$rndId.'">';
        } elseif ($lazyloadContainer && $lowQualityPreview) {

            $img .= '<span class="lazyload-low-quality lazyload-container" style="padding-bottom:' . $this->calculateLazyLoadPaddingBottom($imgOriginalWidth, $imgOriginalHeight) . ';background-image:url(' . $lowQualityUrl . ')">';
        }

        $dataForceLoadData = ($dataForceLoad ? 'data-force-load="true"' : '');


        if ($dataMageInit) {
            $dataMageInit = "data-mage-init='" . $dataMageInit . "'";
        }


        $img .= '<img
                         src="' . ($lazyloadContainer ? $this->getTransparentImage() : $this->getLoadingImage()) . '"
                        ' . $desktopData . '
                        ' . $dataForceLoadData . '
                        ' . $dataMageInit . '
                        class="lazyload ' . $class . '"
                        id="' . $idImage . '"
                        alt="' . $alt . '"
                        />';

        if ($lazyloadContainer) {
            $img .= '</span>';
            $img .= '<style>
                    #lazyload-container_'.$rndId.'{
                        padding-bottom:' . $this->calculateLazyLoadPaddingBottom($imgOriginalWidth, $imgOriginalHeight) . ';
                    }
                    </style>';
        }

        return $img;
    }

    public function calculateLazyLoadPaddingBottom($width, $height) {
        $coeff = '';
        $padding = null;
        if ($width && $height) {
            $coeff = ($height / $width) * 100;
            $padding = round(floatval($coeff), 1) . '%';
        }
        return $padding;
    }

    public function calculateLowQualityImageProportion($originalWidth, $originalHeight, $contraint, $c = 'width') {
        $prop = '';
        if ($originalWidth && $originalHeight && $contraint && $c) {
            switch ($c) {
                case 'width':
                    // $originalWidth : $originalHeight = $contraint : x
                    $prop = ceil((($contraint * $originalHeight) / $originalWidth));
                    break;
                case 'height':
                    // $originalWidth : $originalHeight = x : $contraint
                    $prop = ceil((($contraint * $originalWidth) / $originalHeight));
                    break;
            }
            return $prop;
        }
    }

}
